function ComponentBase(type, id) {
	//type : chart, table, tree, canvas
	
	this.type = type || 'chart';
	this.id = id;
	
	this.time = jui.include("util.time");
	this.active = false;
	
	this.component = {};
	this.data = [];
	this.element = [];
	this.api = {};
	
	this.update = function() {
		if (this.component == null || this.component == undefined) {return;}
		
		if (this.type == 'chart') {
			this.component.axis(0).update(this.data);		
			this.component.setSize(this.element[0].clientWidth, this.element[0].clientHeight - 30);
			this.component.render();
		} else if (this.type == 'speedBar') {
			//
		} else if (this.type == 'topology') {
			//
		} else if (this.type == 'table') {
			this.component.update(this.data);
		} 	
	};
	
	this.resize = function() {		
		if (this.component == null || this.component == undefined) {return;}
		if (this.type == 'chart') {
			if (this.element[0].clientWidth == 0 && this.element[0].clientHeight == 0){
				return;
			}
			this.component.setSize(this.element[0].clientWidth, this.element[0].clientHeight - 30);
			this.component.render();
		} else if (this.type == 'speedBar') {
			//
		} else if (this.type == 'topology') {
			//
		} else {
			//table			
		};		
	};
	
	this.show = function() {		
		if (this.type == 'speedBar') { return; };
		
		this.resize();
		this.element.css('visibility', 'visible');
	};
	
	this.hide = function() {
		if (this.component == null || this.component == undefined) {return;}
		if (this.type == 'speedBar') { return; };
		
		if (this.type == 'chart') {
			this.component.render();
		};
		this.resize();
		this.element.css('visibility', 'hidden');		
	};
};

ComponentBase.prototype.setComponent = function(component) {
	this.component = component;
};

ComponentBase.prototype.init = function() {
	this.active = true;
	if (this.type == 'chart') {
		this.element = angular.element("#"+this.id+" > .box-content");						
	} else if (this.type == 'speedBar') {
			this.element = angular.element("#center"+this.id);
	} else if (this.type == 'topology') {
		this.element = angular.element("#"+this.id+" > .box-content");		
	} else {
		this.element = angular.element("#"+this.id+" > .box-content > .component");
	};
	
	this.resize();
};

ComponentBase.prototype.update = function() {
};

ComponentBase.prototype.resize = function() {
	this.resize();	
};

ComponentBase.prototype.hide = function() {
	this.hide();
};

ComponentBase.prototype.show = function() {
	this.show();
};

ComponentBase.prototype.remove = function() {
	this.component.destroy();
};

ComponentBase.prototype.getDummyData = function() {
};

ComponentBase.prototype.getRealData = function() {	
};

ComponentBase.prototype.pastTimeInterval = function(min, gap) {
	if (gap == undefined){
		gap = 0;
	}
	var clientTime = new Date().getTime();
	var prevTime = clientTime -  (this.time.MINUTE * min) + gap;
	var curTime = clientTime + gap;
	
	return [new Date(prevTime), new Date(curTime)];
	//return [new Date(new Date() - this.time.MINUTE * min), new Date()];
};

ComponentBase.prototype.getTimeToIndex = function() {
	var now = new Date();
    return now.getHours() * 60 + now.getMinutes();
};

ComponentBase.prototype.getZeroData = function(count) {
	var data = [];
    for (var i = 0; i < count; i++) {
        data.push({ w1: ComponentBase.prototype.getRandomValue(0, 0) });
    }
    return data;
};

ComponentBase.prototype.getNumber = function() {
	 return Math.round(Math.random() * 1000) + 1;
};

ComponentBase.prototype.getRandomValue = function(min, max) {
	 return Math.random() * (max - min) + min;
};

ComponentBase.prototype.getTarget = function(arr) {
	var target = [];
	for (var i=0; i<arr.length; i++) {
		target.push(arr[i].name);
	}
	return target;
};

ComponentBase.prototype.getDomain = function(arr, limit) {
	var min = 0;
	var max = 0;
	var temp = [];
	for (var i=0; i<arr.length; i++) {
		temp.push(arr[i].value);
	};
	var max = Math.max.apply(null, temp);	
	if (max < limit) {max = limit;}
	
	return [min, max*1.2];
};

ComponentBase.prototype.getRealTimeDataSource = function(arr) {
	var obj = {};
	for (var i=0; i<arr.length; i++) {
		var prop = arr[i].name;
		var value = arr[i].value;
		if (value == undefined) { 
			value = 0;
		} else {
			value = Math.round(value);
		}
		
		obj[prop] = value;
	}; //for
	return obj;
};

ComponentBase.prototype.getRealTimeTarget = function(arr) {
	return ComponentBase.prototype.getTarget(arr);
};

ComponentBase.prototype.getRealTimeDomain = function(arr) {
	return ComponentBase.prototype.getDomain(arr, 5);
};

ComponentBase.prototype.getEQTarget = function(arr) {
	return ComponentBase.prototype.getTarget(arr);	
};

ComponentBase.prototype.getEQDomain = function(arr) {
	return ComponentBase.prototype.getDomain(arr, 5);	
};

ComponentBase.prototype.getCounterDomain = function(arr) {
	//return ComponentBase.prototype.getDomain(arr, 10);	
	var limit = 5;
	var min = 0;
	var max = 0;
	var temp = [];
	for (var i=0; i<arr.length; i++) {
		temp.push(arr[i]);
	};
	var max = Math.max.apply(null, temp);	
	if (max < limit) {max = limit;}
	
	return [min, max*1.2];
};

ComponentBase.prototype.getXlogDomain = function(arr) {
	//return ComponentBase.prototype.getDomain(arr, 100);	
	var min = 0;
	var max = 0;
	var temp = [];
	for (var i=0; i<arr.length; i++) {
		temp.push(arr[i].elapsed);
	};
	var max = Math.max.apply(null, temp);	
	if (max < 100) {max = 100;}
	
	return [min, max*1.2];
};

ComponentBase.prototype.getToday = function() {
	var date = new Date();
	var year = date.getFullYear();
	var month = (1+date.getMonth());
	month = month>=10? month:'0'+month;
	var day = date.getDate();
	day = day>=10? day:'0'+day;
	return year + '/' + month + '/' + day;
};

ComponentBase.prototype.getYesterday = function() {
	var date = new Date();
	date.setDate(date.getDate()-1);	
	var year = date.getFullYear();
	var month = (1+date.getMonth());
	month = month>=10? month:'0'+month;
	var day = date.getDate();
	day = day>=10? day:'0'+day;
	return year + '/' + month + '/' + day;
};

ComponentBase.prototype.getTomorrow = function() {
	var date = new Date();
	date.setDate(date.getDate()+1);		
	var year = date.getFullYear();
	var month = (1+date.getMonth());
	month = month>=10? month:'0'+month;
	var day = date.getDate();
	day = day>=10? day:'0'+day;
	return year + '/' + month + '/' + day;
};



