function RealTimeXlog(id, $scope, $rootScope, $interval, Data, TickTime) {
	function Component(type, id){
		ComponentBase.apply(this, arguments);
	};	
	inherit(Component, ComponentBase);
	function inherit(C, P) {
		var F = function(){};
		F.prototype = P.prototype;
		C.prototype = new F();
	};
	
	var comp = new Component('chart', id);	
	comp.data = [{level:"normal"}];
	ymaxArr= [];
	comp.yMax = $rootScope.xlogYMax; //10sec
	var chart = new Object();
	
	var configWatch = $scope.$watch('config.visible', function(bool){
		if (!comp.active){
			return;
		}
		if (comp.data.length == 1){
			return;
		}
		if (comp.component == null || comp.component == undefined){
			return;
		}
		
		if (bool) { comp.show(); } 
		else { comp.hide(); }
	}, true);

	createWidget();
	function createWidget() {
		jui.ready(["chart.builder"], function (juiChart) {
			var style = {					
					colors: ["#38c3ff",  "#ffb500", "#ff474c", "#ff474c", "#123456"],
			        backgroundColor: "#00000000",
			        axisBorderColor: "#00000000",
			        axisBorderWidth: 1,
			        axisBorderRadius: 5,
			        gridXFontSize: 10,
			        gridXFontColor: "#c2c6c9",
			        gridYFontSize: 10,
			        gridYFontColor: "#c2c6c9",	    
			        gridXAxisBorderWidth: 1,
			        gridXAxisBorderColor: "#c2c6c9",
			        gridYAxisBorderWidth: 1,
			        gridYAxisBorderColor: "#c2c6c9",
					gridBorderColor:"#464a62",
			        tooltipFontColor:"#fff",
			        tooltipBackgroundColor:"#1b1b1b",
			        lineBorderWidth: 1
			};	
			
			chart = juiChart("#"+id+" > .box-content > .component", {
				padding: { left: 40, top: 20, right: 20, bottom: 30 },
				canvas: true,
			    axis : {
			        x : {
			            type : "date",
			            domain : comp.pastTimeInterval(5, $rootScope.timeGap),
			            interval : 1,
			            realtime: "minutes",
	                    format: "HH:mm",
			            key: "time",
			            line : true
			        },
			        y : {
			            type : "range",
			            domain :  [0, comp.yMax],
			            step : 5,
			            line : true,
			            format : function(d) {
		            		return (d/1000) + "(S)";
			            } 
			        },
			        data : comp.data
			    },
			    brush : [
			    	{
				    	type : "canvas.scatter",
				        size : 3,
				        target : "elapsed",		       
				        symbol: "circle",
			            colors: function (d) {
			            	if($scope.normalFilter && d.error!=2)
			            		return d.error;
			            	else 
			            		return 4;
			            }
		            },
			    	{
			    		type : "canvas.scatter",
			    		size : 6,
			    		target : "errTk",		       
				        symbol: "cross",
			    		colors: function (d) {
			    			if( $scope.errorFilter && d.error==2) 
			    				return d.error;
			            	else 
			            		return 4;
			    		}
			    	}
			    ],
			    widget : [{
			        type : "dragselect",
			        dataType :  "area"
			    }, {
			        type : "tooltip",
			        format : function(data, key) {
    	                return {
    	                    key: data[key],
    	                    value: (data.elapsed) + "ms"
    	                }
    	            }
			    }],
			    event : {
			        "dragselect.end": function(data) {
		        		$scope.openXLogProfile(data);	
			        }
			    },
			    style: style,
			    render: false
			});
			comp.setComponent(chart);
	    	comp.init();   
		});
	}
	
	comp.api.remove = function() {
		configWatch();
		comp.remove();
		comp = null;			
	};
	
	comp.api.update = function(data) {
		if (comp.component == null || comp.component == undefined) {
			createWidget();
			//return;
		}
		if (!comp.active){
			return;
		}
		
		if (comp.data.length == 1 && comp.data[0].time == undefined) {
			comp.data = [];
		}

		var xDomain = comp.pastTimeInterval(5, $rootScope.timeGap);
		comp.component.axis(0).set("x", {domain: xDomain});
		for (var i = 0; i < data.length; i++) {
			data[i].time = new Date(data[i].eTime);
			if(data[i].error==2) {
				data[i].errTk=data[i].elapsed;
				data[i].elapsed=undefined;
			}
            comp.data.push(data[i]);
        }		
		
		var y3=0;
		if (comp.data.length > 0){
	    	var currentTime = xDomain[0].getTime();
	    	
	    	var arrCnt = 0;
	    	while(arrCnt < comp.data.length) {
	    		if (comp.data.length <= arrCnt){
	    			break;
	    		}
	    		var dt1=comp.data[arrCnt];
	    		if (currentTime >= dt1.time.getTime() - 3000){
	    			comp.data.splice(arrCnt , 1);
	    		}else {
	    			if(dt1.errTk){
	    				y3=Math.max(dt1.errTk,y3);
	    			}else if(dt1.elapsed){
	    				y3=Math.max(dt1.elapsed,y3);
	    			}
	    			arrCnt ++;
	    		}
	        }
		}
		
		if(!$scope.autoAxisY) {
			comp.yMax = $rootScope.xlogYMax;
		} else {
			y3=Math.min(170000,y3);
			y3=(Math.floor(y3/5000)+1)*5000;
			comp.yMax=y3;
		}
		var yDomain = [0, comp.yMax];
		comp.component.axis(0).set("y", {domain: yDomain});
		
        comp.update();
	};
	
	comp.api.reload = function() {		
		if (comp.component == null || comp.component == undefined) { return; }
		if (!comp.active) { return; }
		comp.data = [];
		comp.update();		
	};
	
	comp.api.resize = function() {		
		if (!comp.active){
			return;
		}
		comp.resize();		
	};
	
	return comp.api;
};
