
function TopologyView(id, $scope, $rootScope, $interval, Data, TickTime, $timeout) {
	function Component(type, id){
		ComponentBase.apply(this, arguments);
	};	
	inherit(Component, ComponentBase);
	function inherit(C, P) {
		var F = function(){};
		F.prototype = P.prototype;
		C.prototype = new F();
	};
	
	var comp = new Component('topology', id);
	comp.data = {nodes:[], edges:[]};	 
	var chart = new Object();
	var boxElement = null;
	
	var configWatch = $scope.$watch('config.visible', function(bool){
		if (!comp.active){
			return;
		}
		if (bool) { comp.show(); } 
		else { comp.hide(); }
	}, true);
	    
    $timeout(function() {
    	createWidget();
	}, 1000);
		
    var DIR = 'images/topology/';
    var EDGE_LENGTH_MAIN = 150;
    var EDGE_LENGTH_SUB = 50;
    var directionInput = document.getElementById("direction");
    
    var fontStyle = {size:12, color:'white', face:'Nanum Gothic', strokeWidth:0}
    var fontStyleCritical = {size:12, color:'#ff474c', face:'Nanum Gothic', strokeWidth:0}
    var fontStyleWarning = {size:12, color:'#ffb500', face:'Nanum Gothic', strokeWidth:0}
    var fontStyleNormal = {size:12, color:'#38c3ff', face:'Nanum Gothic', strokeWidth:0}
   
    var imageUser = DIR + 'Network-Pipe-icon.png';
    var imageHost = DIR + 'Hardware-Laptop-1-icon.png';
    var imageInstance = DIR + 'Hardware-My-Computer-3-icon.png';	
    
    var imageApache = DIR + 'apache.png';
    var imageJava = DIR + 'java.png';
    var imageJboss = DIR + 'jboss.png';
    var imageLinux = DIR + 'linux.png';
    var imageMariadb = DIR + 'mariadb.png';
    var imageMariaplugin = DIR + 'mariaplugin.png';
    var imageMysql = DIR + 'mysql.png';
    var imageNodejs = DIR + 'nodejs.png';
    var imageOsx = DIR + 'osx.png';
    var imageRedis = DIR + 'redis.png';
    var imageResin = DIR + 'resin.png';
    var imageTomcat = DIR + 'tomcat.png';
    var imageWindows = DIR + 'windows.png';
    
    var lineStyle = {color:'#b2b2b2'}
    
    
	function createWidget() {
		boxElement = $("#"+id+" > .box-content"); 
		var container = $("#"+id+" > .box-content > .component");	      
		var width = boxElement.width();
		var height = boxElement.height();	      
		container.css("width", boxElement.width()+"px");
		container.css("height", boxElement.height()+"px");
		
		var options = {
			width : '100%',
			height : '100%',
			edges: {
				smooth: {
					type: 'cubicBezier',
					forceDirection: 'horizontal',
					roundness: 0.4 
				}
			},
			layout: {
				hierarchical: {
					direction: "UD", //UD, DU, LR, RL
					sortMethod: "directed",
					nodeSpacing: 100,
					levelSeparation:215
	        	}
	        },
	        physics:false
		}; //options
		
		var chart = new vis.Network(container[0], comp.data, options);  		  
		comp.setComponent(chart);
		comp.init();   
	};
	
	comp.api.remove = function() {
		configWatch();
		comp.component.destroy();
		comp.component = null;			
		comp = null;
	};
	
	comp.api.update = function(data) {
		if (data == undefined || data == null || data.length == 0){
			return;
		}
		if (comp.component == null || comp.component == undefined) { createWidget(); return; }
		if (!comp.active){
			return;	
		}
		
		var nodes = [];
	    var edges = [];
	    
	    nodes.push({id: data.groupId, label: data.groupName, level: 0, image: imageUser, shape: 'image', font: fontStyle});
	    var list = data.list
	    var font = fontStyle;
	    for (var i = 0; i < list.length; i++) {
	    	var nodeStatus = list[i].nodeStatus;
	    	var subTyp = list[i].subTyp;
	    	var nodeAlive = list[i].nodeAlive;
	    	
	    	var image = '';
	    	
	    	if(subTyp == 'apache'){
	    		image = imageApache;
	    	}else if(subTyp == 'java'){
	    		image = imageJava;
	    	}else if(subTyp == 'jboss'){
	    		image = imageJboss;
	    	}else if(subTyp == 'linux'){
	    		image = imageLinux;
	    	}else if(subTyp == 'nodejs'){
	    		image = imageNodejs;
	    	}else if(subTyp == 'osx'){
	    		image = imageOsx;
	    	}else if(subTyp == 'redis'){
	    		image = imageRedis;
	    	}else if(subTyp == 'resin'){
	    		image = imageResin;
	    	}else if(subTyp == 'tomcat'){
	    		image = imageTomcat;
	    	}else if(subTyp == 'windows'){
	    		image = imageWindows;
	    	}
	    	
	    	if(nodeAlive){
	    		if(nodeStatus == 'critical'){
		    		font = fontStyleCritical;
		    		image = image.replace('.png','_alert.png');
		    	}else if(nodeStatus == 'warning'){
		    		font = fontStyleWarning;
		    		image = image.replace('.png','_alert.png');
		    	}else if(nodeStatus == 'normal'){
		    		font = fontStyleNormal;
		    	}
	    	}else{
	    		image = image.replace('.png','_inact.png');
	    	}
	    	
	    	
	    	var level = list[i].level;
	    	var nodeLabel = '';
	    	nodeLabel = nodeLabel + list[i].name
	    	if(level == 1){
	    		var value = list[i].compCpu.user + list[i].compCpu.sys + list[i].compCpu.nice + list[i].compCpu.wait; 
	    		nodeLabel = nodeLabel + '\n CPU : ' + value;
	    	}else if(level == 2){
	    		nodeLabel = nodeLabel + '\n ActiveService : ' + list[i].activeServiceCnt;
	    	}
	    	nodes.push({id: list[i].key, label: nodeLabel, level: level, image: image, shape: 'image', font: font});	    	
	    	edges.push({from: list[i].outgoing, to: list[i].key, label: '', length: EDGE_LENGTH_MAIN, font: font, color:lineStyle});
		}
	    
	    comp.data = {nodes:nodes, edges:edges};   
	    comp.component.setData(comp.data);
	};
	
	comp.api.resize = function() {	
		if (!comp.active){
			return;		
		}
		if (comp.component != null && comp.component != undefined) { 
			comp.component.destroy();
			comp.component = null;
			createWidget(); 			 
		}		
	};	
	
	comp.api.reset = function() {
		if (comp.component == null || comp.component == undefined) {
			return;
		}
		if (!comp.active){
			return;		
		}
		if (comp.component.uit == undefined || comp.component.uit == null) {
			return;
		}
		comp.data = {nodes:[], edges:[]};   
	    comp.component.setData(comp.data);
	}
	return comp.api;
};

