app.factory('Data', ['$http', '$mdToast','$mdDialog', '$location', '$rootScope',
    function ($http, $mdToast, $mdDialog, $location, $rootScope) {
		var hostindex = location.href.indexOf(location.host) + location.host.length;
		var serviceBase = location.href.substring(0, hostindex) + "/";
    
        var obj = {};        
        var header = {transformRequest : angular.identity, headers : {'Content-Type' : 'application/x-www-form-urlencoded'}};
        
        function showMdToast(message) {
        	$mdToast.show(
		      $mdToast.simple()
		        .textContent(message)
		        .position('bottom right')
		        .hideDelay(3000)
		    );        	
        };
        
        function alertDialog(title, message) {
        	$mdDialog.show(
    				$mdDialog.alert()
    					.clickOutsideToClose(true)
    					.ariaLabel("Dialog")
    					.title(title)        				
    			        .textContent(message)
    			        .ok('닫기') 
    			        .targetEvent(event)
    			    );             	
        };
        
        function alertOkEventDialogCallback(){
        	return;
		};
        
        function serializeData(data) {
        	if (!angular.isObject(data)) {
        		return ((data==null)? "":data.toString());
        	}
        	var buffer = [];
        	for (var name in data) {
        		if (!data.hasOwnProperty(name)) {
        			continue;
        		}
        		
        		var value = data[name];
        		buffer.push (
        				encodeURIComponent(name) + "=" +
        				encodeURIComponent((value==null)? "":value)
        		);
        	}
        	
        	var source = buffer.join("&").replace(/%20/g, "+");
        	return source;        	
        };
        
        
        
        obj.get = function (q) {
            return $http.get(serviceBase + q, header).then(function (results) {
                return results;
            });
        };

        obj.post = function (q, object) {
            return $http.post(serviceBase + q, serializeData(object), header).then(function (results) {
                return results;
            });
        };
        
        obj.jsonPost = function (q, object) {
            $http({
    			url : serviceBase + q,
    			data : JSON.stringify(object),
    			method : 'POST',
    			headers : {'Content-Type' : 'application/json'}
    		}).success(function(results){
    			return results;
    		}).error(function(data){
    			return data;
    		})
        };
        
        obj.fileDownload = function (q, object, fileName) {
        	$http({
    			url : serviceBase + q,
    			data : serializeData(object), 
    			method : 'POST',
    			headers : {'Content-Type' : 'application/x-www-form-urlencoded'},
    			responseType : 'arraybuffer' 
    		}).success(function(data){
    			var file = new Blob([data],{type:'application/json'});
    			
    			var fileURL = URL.createObjectURL(file);
    			var a = document.createElement('a');
    			a.href = fileURL;
    			a.target = '_blank';
    			a.download = fileName;
    			document.body.appendChild(a);
    			a.click();
    			
    		}).error(function(data){
    		})
        };
        
        obj.fileDownload = function (q, object, fileName, successCallback, errorCallback) {
        	$http({
    			url : serviceBase + q + '.ajax',
    			data : serializeData(object), 
    			method : 'POST',
    			headers : {'Content-Type' : 'application/x-www-form-urlencoded'},
    			responseType : 'arraybuffer' 
    		}).success(function(data){
    			var file = new Blob([data],{type:'application/json'});
    			
    			var fileURL = URL.createObjectURL(file);
    			var a = document.createElement('a');
    			a.href = fileURL;
    			a.target = '_blank';
    			a.download = fileName;
    			document.body.appendChild(a);
    			a.click();
    			if (typeof successCallback !== "undefined") {
    				successCallback();    				
    			}
    			
    		}).error(function(data){
    			errorCallback();
    		})
        };
        
        obj.put = function (q, object) {
            return $http.put(serviceBase + q, serializeData(object), header).then(function (results) {
                return results;
            });
        };

        obj.delete = function (q) {
            return $http.delete(serviceBase + q, header).then(function (results) {
                return results;
            });
        };

        obj.ajax = function(httpObj) {
            if( !httpObj.errorCallback ) {
            	httpObj.errorCallback =  function(xhr, textStatus, errorThrown, errorCallback) {
            		if (xhr  == null){
            			alertDialog("오류", "네트워크 상태가 원할하지 않습니다.");
            		}else{
            			alertDialog("요청응답오류", "오류가 발생하였습니다. 잠시후 다시 사용하시기 바랍니다.");
            		}
                };
            }

            if( !httpObj.successCallback ) {
            	httpObj.successCallback =  function() {};
            }
            
            if( httpObj.param ) {
                httpObj.queryString = $.param(httpObj.param, true);
            }

            function doRequest(httpObj) {
                $http({
                	url: serviceBase + httpObj.url + ".ajax",
                	method : 'POST',
                    dataType : "json",
                    headers : {'Content-Type' : 'application/x-www-form-urlencoded'},
                    data: httpObj.queryString,
                    withCredentials: false
                }).success(function(data, status) {
                	if ( (location.href).indexOf("login") != -1 && $rootScope.authenticated) {
            			$rootScope.authenticated = false;
            			alertDialog("알림", "세션이 만료 되었습니다. 다시 로그인 하세요.");
            		}
                	if (status == '299') {
                		$location.path("/login");
                		return;
                	}
                	
                	/*if (data.result) {
                		if (data.result != '1') {
                			if(data.resultMsg){
                				alertDialog("알림", data.resultMsg);
                				return;
                			}
                        }
                	}*/
                	 
                	httpObj.successCallback(data, status);
                }).error(httpObj.errorCallback);
            }
            
            doRequest(httpObj);
            
        };
        
        return obj;
    }]);