jui.define("chart.widget.customTooltip", [ "jquery", "util.color","util.base" ], function(_) {
	/**
	 * Tooltip으로 보여줘야할 데이터가 Array일 경우 멀티라인으로 보여주게하는 Tooltip
	 */
	var cnt=0;
	var CustomTooltipWidget = function(chart, axis, widget){
		var self = this;
		var g, text, rect;
        var padding = 7, anchor = 7, textY = 14;
        var tspan = []; // 멀티라인일 경우, 하위 노드 캐시
        
        function setMessage(index, message) {
            if(!tspan[index]) {
                var elem = document.createElementNS("http://www.w3.org/2000/svg", "tspan");
                text.element.appendChild(elem);
                tspan[index] = elem;
            }
            
            tspan[index].textContent = message;
        }
        
        function getFormat(key, value, data) {
            if(typeof(widget.format) == "function" && widget.multiLine == false) {
                return self.format(key, value, data);
            }else {
                if (!value) {
                    return key;
                }

                return key + ": " + self.format(value);
            }
        }

        function printTooltip(obj) {
        	if(obj.dataKey && widget.all === false && widget.multiLine == false) {
                var k = obj.dataKey,
                    d = (obj.data != null) ? obj.data[k] : null;

                // 위젯 포지션에 따른 별도 처리
                if(widget.orient == "bottom") {
                    text.attr({ y: textY + anchor });
                }

                // 툴팁 값 설정
                var message = getFormat(k, d, obj.data);
                setMessage(0, message);

                text.attr({ "text-anchor": "middle" });
            } else {
            	if(widget.multiLine == true){
            		for(var i = 0; i<obj.data[widget.arrayInfo.name].length;i++){
            			var arr = obj.data[widget.arrayInfo.name][i];
            			var key = arr[widget.arrayInfo.key];
            			var value = arr[widget.arrayInfo.value];
            			 
	            			x = padding,
	                        y = (textY * i) + (padding * 2),
	                        d = (key != null) ? key : null;
	            			
	            			// 위젯 포지션에 따른 별도 처리
	                        if(widget.orient == "bottom") {
	                            y = y + anchor;
	                        }

	                        var message = getFormat(key, value, obj.data);
	                        setMessage(i, message);

	                        tspan[i].setAttribute("x", x);
	                        tspan[i].setAttribute("y", y);
            		}
            	}else{
            		var brush = obj.brush;

                    for(var i = 0; i < brush.target.length; i++) {
                        var key = brush.target[i],
                            x = padding,
                            y = (textY * i) + (padding * 2),
                            d = (obj.data != null) ? obj.data[key] : null;

                        // 위젯 포지션에 따른 별도 처리
                        if(widget.orient == "bottom") {
                            y = y + anchor;
                        }

                        var message = getFormat(key, d, obj.data);
                        setMessage(i, message);

                        tspan[i].setAttribute("x", x);
                        tspan[i].setAttribute("y", y);
                    }
            	}
            	

                text.attr({ "text-anchor": "inherit" });
            }
        }
        
        function existBrush(index) {
            var list = self.getIndexArray(self.widget.brush);

            return ($.inArray(index, list) == -1) ? false : true;
        }

        function getColorByKey(obj) {
            var targets = obj.brush.target;

            for(var i = 0; i < targets.length; i++) {
                if(targets[i] == obj.dataKey) {
                    return ColorUtil.lighten(self.chart.color(i, obj.brush.colors, targets));
                }
            }

            return chart.theme("tooltipBorderColor");
        }
        
        this.drawBefore = function() {
            g = chart.svg.group({
                visibility: "hidden"
            }, function() {
                rect = chart.svg.polygon({
                    fill: chart.theme("tooltipBackgroundColor"),
                    "fill-opacity": chart.theme("tooltipBackgroundOpacity"),
                    stroke: chart.theme("tooltipBorderColor"),
                    "stroke-width": 1
                });

                text = chart.text({
                    "font-size": chart.theme("tooltipFontSize"),
                    "fill": chart.theme("tooltipFontColor"),
                    y: textY
                });
            });
        }
       
        
        this.draw = function() {
            var isActive = false,
                w, h; 
            this.on("mouseover", function(obj, e) {
            	if(isActive || !existBrush(obj.brush.index)){ return};
                if(!obj.dataKey && !obj.data){ return};

                // 툴팁 텍스트 출력
                printTooltip(obj);
                
                var size = text.size();
                w = size.width + (padding * 2);
                h = size.height + padding;

                text.attr({ x: w / 2 });
                rect.attr({ points: self.balloonPoints(widget.orient, w, h, anchor) });
                g.attr({ visibility: "visible" });

                isActive = true;
            });

            this.on("mousemove", function(obj, e) {
                if(!isActive){ return};

                var x = e.bgX - (w / 2),
                    y = e.bgY - h - anchor - (padding / 2);

                if(widget.orient == "left" || widget.orient == "right") {
                    y = e.bgY - (h / 2) - (padding / 2);
                }

                if(widget.orient == "left") {
                    x = e.bgX - w - anchor;
                } else if(widget.orient == "right") {
                    x = e.bgX + anchor;
                } else if(widget.orient == "bottom") {
                    y = e.bgY + (anchor * 2);
                }

                g.translate(x, y);
            });

            this.on("mouseout", function(obj, e) {
                if(!isActive){ return};
                tspan =[];
                while(text.element.firstChild){
                	text.element.removeChild(text.element.firstChild);
                }
                
                g.attr({ visibility: "hidden" }); 
                isActive = false;
            });

            return g;
        }
	}
	CustomTooltipWidget.setup = function() {
		return {
			multiLine:false,
			/** @cfg {Object} [area={}]  set area(name, key, value) of axis */
			arrayInfo:{}
		};
	}
	return CustomTooltipWidget;
}, "chart.widget.tooltip");

/**
 * @class chart.widget.xyTitle
 * implements xyTitle widget
 * @extends chart.widget.core
 * @alias XYTitle
 * @requires util.base,util.math
 * x축,y축 타이틀 
 */
jui.define("chart.widget.xyTitle", [ "util.base","util.math" ], function(_,math) {
	
	var XYTitle = function(chart, axis, widget){
		 var x = 0, y = 0,x2 = 0, y2 = 0, anchor = "middle";
		 
		
		 this.drawBefore = function() {
			 x = 0+ chart.padding("left") - 20 ;
			 y = chart.area("y2")/2;
			 x2 = chart.area("x2")/2;
			 y2 = chart.area("y2")+ chart.padding("bottom") - 10;
		 }
		 
		 this.draw = function() {
			 var group = chart.svg.group();
			 var xTitleObj = chart.svg.getTextSize(widget.xTitle);
			 var yTitleObj = chart.svg.getTextSize(widget.yTitle);
			 
			 
			 var half_text_width = xTitleObj.width / 2,
             	 half_text_height = xTitleObj.height / 2;
			 
			 var yText =  chart.text({
                x : x,
                y : y,
                "text-anchor" : anchor,
                "fill" : widget.color ,
                "font-size" : widget.size,
                "font-weight" :  "bold"
            }, widget.yTitle);
			 
			 var xText =  chart.text({
                x : x2,
                y : y2,
                "text-anchor" : anchor,
                "fill" : widget.color ,
                "font-size" : widget.size ,
                "font-weight" : "bold"
            }, widget.xTitle);
			 yText.rotate(widget.rotate, x + widget.dx, y + widget.dy)
			 group.append(xText);
			 group.append(yText);
			 
			 return group; 
			 
		 }
		
		
	}
	 
	XYTitle.setup = function(){
		return {
			rotate:0,
			dx:0,
			dy:0,
			color:"#333", 
			size:18,
			xTitle:"",
			yTitle:""
		};
	}
	
	return XYTitle;
},"chart.widget.core");

jui.define("chart.widget.customDragselect", [ "util.base" ], function(_) {

    /**
     * @class chart.widget.dragselect
     * @extends chart.widget.core
     * @alias DragSelectWidget
     * @requires util.base
     *
     */
    var DragSelectWidget = function() {
        var self = this;

        function setDragEvent(brush, thumb) {
            var axis = self.chart.axis(brush.axis),
                isMove = false,
                mouseStartX = 0,
                mouseStartY = 0,
                thumbWidth = 0,
                thumbHeight = 0,
                startValueX = 0,
                startValueY = 0;

            self.on("axis.mousedown", function(e) {
                if(isMove){ return;}

                isMove = true;
                mouseStartX = e.bgX;
                mouseStartY = e.bgY;
                startValueX = axis.x.invert(e.chartX);
                startValueY = axis.y.invert(e.chartY);

                self.chart.emit("dragselect.start");
            }, brush.axis);

            self.on("axis.mousemove", function(e) {
                if(!isMove){
                	return;
                }

                thumbWidth = e.bgX - mouseStartX;
                thumbHeight = e.bgY - mouseStartY

                thumb.attr({
                    width: (thumbWidth >= 0) ? thumbWidth : Math.abs(thumbWidth),
                    height: (thumbHeight >= 0) ? thumbHeight : Math.abs(thumbHeight)
                });

                thumb.translate(
                    (thumbWidth >= 0) ? mouseStartX : mouseStartX + thumbWidth,
                    (thumbHeight >= 0) ? mouseStartY : mouseStartY + thumbHeight
                );
            }, brush.axis);

            self.on("axis.mouseup", endZoomAction, brush.axis);
            self.on("chart.mouseup", endZoomAction);
            self.on("bg.mouseup", endZoomAction);

            function endZoomAction(e) {
                isMove = false;
                if(thumbWidth == 0 || thumbHeight == 0){
                	self.chart.emit("dragselect.reset");
                	return;
                }

                searchDataInDrag(axis.x.invert(e.chartX), axis.y.invert(e.chartY));
                resetDragStatus();
            }

            function searchDataInDrag(endValueX, endValueY) {
                // x축 값 순서 정하기
                if(startValueX > endValueX) {
                    var temp = startValueX;
                    startValueX = endValueX;
                    endValueX = temp;
                }

                // y축 값 순서 정하기
                if(startValueY > endValueY) {
                    var temp = startValueY;
                    startValueY = endValueY;
                    endValueY = temp;
                }

                if(self.widget.dataType == "area") {
                    emitDragArea(startValueX, startValueY, endValueX, endValueY);
                } else {
                    emitDataList(startValueX, startValueY, endValueX, endValueY);
                }
            }

            function emitDataList(startValueX, startValueY, endValueX, endValueY) {
                var xType = axis.x.type,
                    yType = axis.y.type,
                    datas = axis.data,
                    targets = brush.target,
                    dataInDrag = [];

                // 해당 브러쉬의 데이터 검색
                for(var i = 0; i < datas.length; i++) {
                    var d = datas[i];

                    for(var j = 0; j < targets.length; j++) {
                        var v = d[targets[j]];

                        // Date + Range
                        if(xType == "date" && yType == "range") {
                            var date = d[axis.get("x").key];

                            if(_.typeCheck("date", date)) {
                                if( (date.getTime() >= startValueX.getTime() && date.getTime() <= endValueX.getTime()) &&
                                    (v >= startValueY && v <= endValueY) ) {
                                    dataInDrag.push(getTargetData(i, targets[j], d));
                                }
                            }
                        } else if(xType == "range" && yType == "date") {
                            var date = d[axis.get("y").key];

                            if(_.typeCheck("date", date)) {
                                if( (date.getTime() >= startValueY.getTime() && date.getTime() <= endValueY.getTime()) &&
                                    (v >= startValueX && v <= endValueX) ) {
                                    dataInDrag.push(getTargetData(i, targets[j], d));
                                }
                            }
                        }

                        // Block + Range
                        if(xType == "block" && yType == "range") {
                            if( (i >= startValueX - 1 && i <= endValueX - 1) &&
                                (v >= startValueY && v <= endValueY)) {
                                dataInDrag.push(getTargetData(i, targets[j], d));
                            }
                        } else if(xType == "range" && yType == "block") {
                            if( (i >= startValueY - 1 && i <= endValueY - 1) &&
                                (v >= startValueX && v <= endValueX) ) {
                                dataInDrag.push(getTargetData(i, targets[j], d));
                            }
                        }
                    }
                }

                function getTargetData(index, key, data) {
                    return {
                        brush: brush,
                        dataIndex: index,
                        dataKey: key,
                        data: data
                    };
                }

                self.chart.emit("dragselect.end", [ dataInDrag ]);
            }

            function emitDragArea(startValueX, startValueY, endValueX, endValueY) {
                self.chart.emit("dragselect.end", [ {
                    x1: startValueX,
                    y1: startValueY,
                    x2: endValueX,
                    y2: endValueY
                } ]);
            }

            function resetDragStatus() { // 엘리먼트 및 데이터 초기화
                isMove = false;
                mouseStartX = 0;
                mouseStartY = 0;
                thumbWidth = 0;
                thumbHeight = 0;
                startValueX = 0;
                startValueY = 0;

                thumb.attr({
                    width: 0,
                    height: 0
                });
            }
        }

        this.drawSection = function(brush) {
            return this.chart.svg.group({}, function() {
                var thumb = self.chart.svg.rect({
                    width: 0,
                    height: 0,
                    stroke: self.chart.theme("dragSelectBorderColor"),
                    "stroke-width": self.chart.theme("dragSelectBorderWidth"),
                    fill: self.chart.theme("dragSelectBackgroundColor"),
                    "fill-opacity": self.chart.theme("dragSelectBackgroundOpacity")
                });

                setDragEvent(brush, thumb);
            });
        }

        this.draw = function() {
            var g = this.chart.svg.group(),
                bIndex = this.widget.brush,
                bIndexes = (_.typeCheck("array", bIndex) ? bIndex : [ bIndex ]);

            for(var i = 0; i < bIndexes.length; i++) {
                var brush = this.chart.get("brush", bIndexes[i]);

                if(brush != null) {
                    g.append(this.drawSection(brush));
                }
            }

            return g;
        }
    }

    DragSelectWidget.setup = function() {
        return {
            brush: [ 0 ],
            dataType: "list" // or area
        }
    }

    return DragSelectWidget;
}, "chart.widget.core");

jui.define("chart.widget.dashbdDragselect", [ "util.base" ], function(_) {
   var DragSelectWidget = function() {
        var self = this;

        function setDragEvent(brush, thumb) {
            var axis = self.chart.axis(brush.axis),
                isMove = false,
                mouseStartX = 0,
                mouseStartY = 0,
                thumbWidth = 0,
                thumbHeight = 0,
                gapWidth = 0,
                gapHeight = 0,
                startValueX = 0,
                startValueY = 0;

            self.on("axis.mousedown", function(e) {
                if(isMove){ return; }

                isMove = true;
                mouseStartX = e.bgX;
                mouseStartY = e.bgY;
                startValueX = axis.x.invert(e.chartX);
                startValueY = axis.y.invert(e.chartY);

                //console.log(mouseStartX, mouseStartY, startValueX, startValueY);
                
                self.chart.emit("dragselect.start");
            }, brush.axis);
            
            self.on("axis.mousemove", function(e) {
                if(!isMove){ return; }

                //console.log('axis : ', e.bgX , ',', e.bgX);

                thumbWidth = e.bgX - mouseStartX;
                thumbHeight = e.bgY - mouseStartY

                thumb.attr({
                    width: (thumbWidth >= 0) ? thumbWidth : Math.abs(thumbWidth),
                    height: (thumbHeight >= 0) ? thumbHeight : Math.abs(thumbHeight)
                });

                thumb.translate(
                    (thumbWidth >= 0) ? mouseStartX : mouseStartX + thumbWidth,
                    (thumbHeight >= 0) ? mouseStartY : mouseStartY + thumbHeight
                );
            }, brush.axis);

            self.on("axis.mouseup", endZoomAction, brush.axis);
            self.on("chart.mouseup", endZoomAction);
            self.on("bg.mouseup", endZoomAction);
            
            document.onmouseup = function(e){
                if(!isMove){ return; }
                endZoomAction(e);
            };

         /*   document.onmousemove = function(e) {
                if(!isMove) return;

                if(!isNaN(e.chartX)) thumbWidth = e.x - mouseStartX;
                if(!isNaN(e.chartY))  thumbHeight = e.y - mouseStartY;

                thumb.attr({
                    width: (thumbWidth >= 0) ? thumbWidth : Math.abs(thumbWidth),
                    height: (thumbHeight >= 0) ? thumbHeight : Math.abs(thumbHeight)
                });

                thumb.translate(
                    (thumbWidth >= 0) ? mouseStartX : mouseStartX + thumbWidth,
                    (thumbHeight >= 0) ? mouseStartY : mouseStartY + thumbHeight
                );
            };*/

            function endZoomAction(e) {
                isMove = false;
                if(thumbWidth == 0 || thumbHeight == 0){ return; }

                searchDataInDrag(axis.x.invert(e.chartX), axis.y.invert(e.chartY));
                resetDragStatus();
            }

            function searchDataInDrag(endValueX, endValueY) {
                // x축 값 순서 정하기
                if(startValueX > endValueX) {
                    var temp = startValueX;
                    startValueX = endValueX;
                    endValueX = temp;
                }

                // y축 값 순서 정하기
                if(startValueY > endValueY) {
                    var temp = startValueY;
                    startValueY = endValueY;
                    endValueY = temp;
                }

                if(self.widget.dataType == "area") {
                    emitDragArea(startValueX, startValueY, endValueX, endValueY);
                } else {
                    emitDataList(startValueX, startValueY, endValueX, endValueY);
                }
            }

            function emitDataList(startValueX, startValueY, endValueX, endValueY) {
                var xType = axis.x.type,
                    yType = axis.y.type,
                    datas = axis.data,
                    targets = brush.target,
                    dataInDrag = [];

                // 해당 브러쉬의 데이터 검색
                for(var i = 0; i < datas.length; i++) {
                    var d = datas[i];

                    for(var j = 0; j < targets.length; j++) {
                        var v = d[targets[j]];

                        // Date + Range
                        if(xType == "date" && yType == "range") {
                            var date = d[axis.get("x").key];

                            if(_.typeCheck("date", date)) {
                                if( (date.getTime() >= startValueX.getTime() && date.getTime() <= endValueX.getTime()) &&
                                    (v >= startValueY && v <= endValueY) ) {
                                    dataInDrag.push(getTargetData(i, targets[j], d));
                                }
                            }
                        } else if(xType == "range" && yType == "date") {
                            var date = d[axis.get("y").key];

                            if(_.typeCheck("date", date)) {
                                if( (date.getTime() >= startValueY.getTime() && date.getTime() <= endValueY.getTime()) &&
                                    (v >= startValueX && v <= endValueX) ) {
                                    dataInDrag.push(getTargetData(i, targets[j], d));
                                }
                            }
                        }

                        // Block + Range
                        if(xType == "block" && yType == "range") {
                            if( (i >= startValueX - 1 && i <= endValueX - 1) &&
                                (v >= startValueY && v <= endValueY)) {
                                dataInDrag.push(getTargetData(i, targets[j], d));
                            }
                        } else if(xType == "range" && yType == "block") {
                            if( (i >= startValueY - 1 && i <= endValueY - 1) &&
                                (v >= startValueX && v <= endValueX) ) {
                                dataInDrag.push(getTargetData(i, targets[j], d));
                            }
                        }
                    }
                }

                function getTargetData(index, key, data) {
                    return {
                        brush: brush,
                        dataIndex: index,
                        dataKey: key,
                        data: data
                    };
                }

                self.chart.emit("dragselect.end", [ dataInDrag ]);
            }

            function emitDragArea(startValueX, startValueY, endValueX, endValueY) {
                self.chart.emit("dragselect.end", [ {
                    x1: startValueX,
                    y1: startValueY,
                    x2: endValueX,
                    y2: endValueY
                } ]);
            }

            function resetDragStatus() { // 엘리먼트 및 데이터 초기화
                isMove = false;
                mouseStartX = 0;
                mouseStartY = 0;
                thumbWidth = 0;
                thumbHeight = 0;
                startValueX = 0;
                startValueY = 0;

                thumb.attr({
                    width: 0,
                    height: 0
                });
            }
        }

        this.drawSection = function(brush) {
            return this.chart.svg.group({}, function() {
                var thumb = self.chart.svg.rect({
                    width: 0,
                    height: 0,
                    stroke: self.chart.theme("dragSelectBorderColor"),
                    "stroke-width": self.chart.theme("dragSelectBorderWidth"),
                    fill: self.chart.theme("dragSelectBackgroundColor"),
                    "fill-opacity": self.chart.theme("dragSelectBackgroundOpacity")
                });

                setDragEvent(brush, thumb);
            });
        }

        this.draw = function() {
            var g = this.chart.svg.group(),
                bIndex = this.widget.brush,
                bIndexes = (_.typeCheck("array", bIndex) ? bIndex : [ bIndex ]);

            for(var i = 0; i < bIndexes.length; i++) {
                var brush = this.chart.get("brush", bIndexes[i]);

                if(brush != null) {
                    g.append(this.drawSection(brush));
                }
            }

            return g;
        }
    }

    DragSelectWidget.setup = function() {
        return {
            brush: [ 0 ],
            dataType: "list" // or area
        }
    }

    return DragSelectWidget;
}, "chart.widget.core");


jui.define("chart.brush.pie.customPie", [ "util.base", "util.math", "util.color" ], function(_, math, ColorUtil) {
	
	
	var CustomPieBrush = function() {
		var self = this, textY = 3;
        var g;
        var cache_active = {};
        
        
        this.sorting = function(target,max,outerRadius,data,centerY){
    		var all = 360;
    		var c = this.chart,r = outerRadius * c.theme("pieOuterLineRate");
    		var tyArr = new Array();
    		var rateAtt = new Array();
    		var centerAngleArr = new Array(); 
    		var startAngle =0;
    		var size = 12;
    		for (var i = 0; i < target.length; i++) {
    			var value = data[target[i]],
                    endAngle = all * (value / max),
                    centerAngle = startAngle + (endAngle / 2) - 90;
    				
    				ty = centerY + (Math.sin(math.radian(centerAngle)) * r);
    				tyArr.push(ty);
    				rateAtt.push(endAngle/all*100);
    				centerAngleArr.push(centerAngle);
    				
    				startAngle+=endAngle;
    		}
    		recurFunc();
    		function recurFunc(){
    			for(var i = 0 ; i< target.length; i++){
    				if(1<target.length && i+1 <= target.length){
    					
    					if(centerAngleArr[i]+90 < 90){
    						var tyDiff = tyArr[i] - tyArr[i+1];
    						if(Math.abs(tyDiff)<size){
    							tyArr[i] = tyArr[i+1]-size;
    							recurFunc();
    						}
    					}else if(centerAngleArr[i]+90 > 90 && centerAngleArr[i]+90 < 180){
    						var tyDiff = tyArr[i] - tyArr[ i-1];
    						if(Math.abs(tyDiff)<size){
    							tyArr[i] = tyArr[i-1]+size;
    							recurFunc();
    						}
    					}else if(centerAngleArr[i]+90 > 180 && centerAngleArr[i]+90 < 270){
    						var tyDiff = tyArr[i] - tyArr[i+1];
    						if(Math.abs(tyDiff)<size){ 
    							tyArr[i] = tyArr[i+1]+size;
    							recurFunc();
    						}
    					}else if(centerAngleArr[i]+90 > 270 && centerAngleArr[i]+90 < 360){
    						var tyDiff = tyArr[i] - tyArr[i-1];
    						if(Math.abs(tyDiff)<size){
    							tyArr[i] = tyArr[i-1]-size;
    							recurFunc();
    						}
    					}
    				}else if(i == target.length-1){
    					var tyDiff = tyArr[i] - tyArr[i-1];
						if(Math.abs(tyDiff)<size){
							tyArr[i] = tyArr[i-1]-size;
							recurFunc();
						}
    				}
    			}
    		}
    		return tyArr;
    	}
        
        
        this.drawText = function(centerX, centerY, centerAngle, outerRadius, text ,index ,tyArr) {
            var c = this.chart,
                dist = c.theme("pieOuterLineSize"),
                r = outerRadius * c.theme("pieOuterLineRate"),
                cx = centerX + (Math.cos(math.radian(centerAngle)) * outerRadius),
                cy = centerY + (Math.sin(math.radian(centerAngle)) * outerRadius),
                tx = centerX + (Math.cos(math.radian(centerAngle)) * r),
                ty = tyArr[index],
                isLeft = (centerAngle + 90 > 180) ? true : false,
                ex = (isLeft) ? tx - dist : tx + dist;

            return c.svg.group({}, function() {
                var path = c.svg.path({
                    fill: "transparent",
                    stroke: c.theme("pieOuterLineColor"),
                    "stroke-width": 0.7
                });

                path.MoveTo(cx, cy)
                    .LineTo(tx, ty)
                    .LineTo(ex, ty);

                c.text({
                    "font-size": c.theme("pieOuterFontSize"),
                    fill: c.theme("pieOuterFontColor"),
                    "text-anchor": (isLeft) ? "end" : "start",
                    y: textY
                }, text).translate(ex + (isLeft ? -3 : 3), ty);
            });
        }
        
		this.drawUnit = function (index, data, g) {
			var obj = this.axis.c(index);

			var width = obj.width,
                height = obj.height,
                x = obj.x,
                y = obj.y,
                min = width;

			if (height < min) {
				min = height;
			}

			// center
			var centerX = width / 2 + x,
                centerY = height / 2 + y,
                outerRadius = min / 2;

			var target = this.brush.target,
                active = this.brush.active,
				all = 360,
				startAngle = 0,
				max = 0;

			for (var i = 0; i < target.length; i++) {
				max += data[target[i]];
			}

            startAngle = 0;
            
            var tyArr = this.sorting(target,max,outerRadius,data,centerY);
            
			for (var i = 0; i < target.length; i++) {
                var value = data[target[i]],
                    endAngle = all * (value / max),
                    centerAngle = startAngle + (endAngle / 2) - 90,
                    pie = this.drawPie(centerX, centerY, outerRadius, startAngle, endAngle, this.color(i));

                // 설정된 키 활성화
                if (active == target[i] || $.inArray(target[i], active) != -1) {
                    this.setActiveEvent(pie, centerX, centerY, centerAngle);
                    cache_active[centerAngle] = true;
                }

                // 활성화 이벤트 설정
                if (this.brush.activeEvent != null) {
                    (function(p, cx, cy, ca) {
                        p.on(self.brush.activeEvent, function(e) {
                            if(!cache_active[ca]) {
                                self.setActiveEvent(p, cx, cy, ca);
                                cache_active[ca] = true;
                            } else {
                                p.translate(cx, cy);
                                cache_active[ca] = false;
                            }
                        });

                        p.attr({ cursor: "pointer" });
                    })(pie, centerX, centerY, centerAngle);
                }
                
                if (this.brush.showText) {
                	
                    var text = this.getFormatText(target[i], value, max),
                        elem = this.drawText(centerX, centerY, centerAngle, outerRadius, text, i ,tyArr);

                    this.addEvent(elem, index, i);
                    g.append(elem);
                }

                self.addEvent(pie, index, i);
                g.append(pie);

				startAngle += endAngle;
			}
		}
	}
	
	
	return CustomPieBrush;
},"chart.brush.pie");

jui.define("chart.widget.customCross", [ "util.base" ], function(_) {

    /**
     * @class chart.widget.cross
     * @extends chart.widget.core
     * @alias CoreWidget
     * @requires util.base
     *
     */
    var CrossWidget = function(chart, axis, widget) {
        var self = this;
        var tw = 100, th = 18, ta = tw / 20; // 툴팁 넓이, 높이, 앵커 크기
        var pl = 0, pt = 0; // 엑시스까지의 여백
        var g, xline, yline, xTooltip, yTooltip;
        var tspan = [];

        function printTooltip(index, text, message) {
            if(!tspan[index]) {
                var elem = document.createElementNS("http://www.w3.org/2000/svg", "tspan");
                text.element.appendChild(elem);
                tspan[index] = elem;
            }

            tspan[index].textContent = message;
        }

        this.drawBefore = function() {
            // 위젯 옵션에 따라 엑시스 변경
            axis = this.chart.axis(widget.axis);

            // 엑시스 여백 값 가져오기
            pl = chart.padding("left") + axis.area("x");
            pt = chart.padding("top") + axis.area("y");

            g = chart.svg.group({
                visibility: "hidden"
            }, function() {
                // 포맷 옵션이 없을 경우, 툴팁을 생성하지 않음
                if(_.typeCheck("function", widget.yFormat)) {
                    xline = chart.svg.line({
                        x1: 0,
                        y1: 0,
                        x2: axis.area("width"),
                        y2: 0,
                        stroke: chart.theme("crossBorderColor"),
                        "stroke-width": chart.theme("crossBorderWidth"),
                        opacity: chart.theme("crossBorderOpacity")
                    });

                    yTooltip = chart.svg.group({}, function () {
                        chart.svg.polygon({
                            fill: chart.theme("crossBalloonBackgroundColor"),
                            "fill-opacity": chart.theme("crossBalloonBackgroundOpacity"),
                            points: self.balloonPoints("left", tw, th, ta)
                        });

                        chart.text({
                            "font-size": chart.theme("crossBalloonFontSize"),
                            "fill": chart.theme("crossBalloonFontColor"),
                            "text-anchor": "middle",
                            x: tw / 2,
                            y: 12
                        });
                    }).translate(-(tw + ta), 0);
                }

                if(_.typeCheck("function", widget.xFormat)) {
                    yline = chart.svg.line({
                        x1: 0,
                        y1: 0,
                        x2: 0,
                        y2: axis.area("height"),
                        stroke: chart.theme("crossBorderColor"),
                        "stroke-width": chart.theme("crossBorderWidth"),
                        opacity: chart.theme("crossBorderOpacity")
                    });

                    xTooltip = chart.svg.group({}, function () {
                        chart.svg.polygon({
                            fill: chart.theme("crossBalloonBackgroundColor"),
                            "fill-opacity": chart.theme("crossBalloonBackgroundOpacity"),
                            points: self.balloonPoints("bottom", tw, th, ta)
                        });

                        chart.text({
                            "font-size": chart.theme("crossBalloonFontSize"),
                            "fill": chart.theme("crossBalloonFontColor"),
                            "text-anchor": "middle",
                            x: tw / 2,
                            y: 17
                        });
                    }).translate(0, axis.area("height") + ta);
                }
            }).translate(pl, pt);
        }

        this.draw = function() {
            this.on("axis.mouseover", function(e) {
                g.attr({ visibility: "visible" });
            }, widget.axis);

            this.on("axis.mouseout", function(e) {
                g.attr({ visibility: "hidden" });
            }, widget.axis);

            this.on("axis.mousemove", function(e) {
                var left = e.bgX - pl,
                    top = e.bgY - pt + 2;

                if(xline) {
                    xline.attr({
                        y1: top,
                        y2: top
                    });
                }

                if(yline) {
                    yline.attr({
                        x1: left,
                        x2: left
                    });
                }

                // 포맷 옵션이 없을 경우, 처리하지 않음
                if(yTooltip) {
                    yTooltip.translate(-(tw + ta), top - (th / 2));

                    var value = axis.y.invert(e.chartY),
                        message = widget.yFormat.call(self.chart, value);
                    printTooltip(0, yTooltip.get(1), message);
                }

                if(xTooltip) {
                    xTooltip.translate(left - (tw / 2), axis.area("height") + ta);

                    var value = axis.x.invert(e.chartX),
                        message = widget.xFormat.call(self.chart, value);
                    printTooltip(1, xTooltip.get(1), message);
                }
            }, widget.axis);

            return g;
        }
    }

    CrossWidget.setup = function() {
        return {
            axis: 0,

            /**
             * @cfg {Function} [xFormat=null] Sets the format for the value on the X axis shown on the tooltip.
             */            
            xFormat: null,
            /**
             * @cfg {Function} [yFormat=null] Sets the format for the value on the Y axis shown on the tooltip.
             */
            yFormat: null
        };
    }

    return CrossWidget;
}, "chart.widget.core");

jui.define("chart.topology.sort.customLinear", [], function() {
    var cache = {};

    function getRandomRowIndex(row_cnt) {
        var row_index = Math.floor(Math.random() * row_cnt);

        if(cache[row_index]) {
            var cnt = 0;
            for(var k in cache) { cnt++; }

            if(cnt < row_cnt) {
                return getRandomRowIndex(row_cnt);
            } else {
                cache = {};
            }
        } else {
            cache[row_index] = true;
        }

        return row_index;
    }

    return function(data, area, space) {
        var xy = [],
            row_cnt = Math.floor(area.height / space),
            col_cnt = Math.floor(area.width / space),
            col_step = Math.floor(col_cnt / data.length) + 1,
            col_index = 0;

        var left = -1,
            right = data.length;

        for(var i = 0; i < data.length; i++) {
            var x = 0, y = 0, index = 0;

            if(i % 2 == 0) {
                x = col_index * space;
                y = getRandomRowIndex(row_cnt) * space;
                col_index += col_step;

                left += 1;
                index = left;
            } else {
                x = (col_cnt - col_index) * space + space;
                y = getRandomRowIndex(row_cnt) * space;

                right -=1;
                index = right;
            }

            xy[index] = {
                x: area.x + x + space,
                y: area.y + y + (space / 2)
            };
        }

        return xy;
    }
});

function chartResize(){  
	var win_width = 0;
	if(jui) {
		var _ = jui.include("util.base");
		if(win_width == $(window).width()) {return};
		
		var call_list = jui.get("chart.builder");
		for(var i = 0; i < call_list.length; i++) {
			var ui_list = call_list[i];
			
			for(var j = 0; j < ui_list.length; j++) {
				if(ui_list[j].isFullSize()) {
					ui_list[j].setSize();
				}
				
				if(!ui_list[j].isRender()) {
					ui_list[j].render(true);
				}
			}
		}
		
		win_width = $(window).width(); 
	}
}

